using System;
using System.Collections.Generic;
using System.Text;

namespace encinsp
{
    // MPEG version codes
    public enum MpegVersion : byte
    {
        Version25 = 0,
        Unknown = 1,
        Version2 = 2,
        Version1 = 3
    }

    // MPEG layer codes
    public enum MpegLayer : byte
    {
        Unknown = 0,
        LayerIII = 1,
        LayerII = 2,
        LayerI = 3
    }

    // Channel mode codes
    public enum MpegChannel : byte
    {
        Stereo = 0,
        JointStereo = 1,
        DualChannel = 2,
        Mono = 3,
        Unknown = 4
    }

    // Extension mode codes (for Joint Stereo)
    public enum JointStereoExtensionMode : byte
    {
        Off = 0,
        IS = 1,
        MS = 2,
        On = 3,
        Unknown = 4
    }

    // Emphasis mode codes
    public enum Emphasis : byte
    {
        None = 0,
        _5015 = 1,
        Unknown = 2,
        CCIT = 3
    }

    // Sample rate codes
    public enum SampleRateLevel : ushort // UInt16
    {
        Level3 = 0,
        Level2 = 1,
        Level1 = 2,
        Unknown = 3
    }

    // Encoder codes
    public enum MpegEncoder
    {
        Unknown,
        Xing,
        FhG,
        LAME,
        Blade,
        GoGo,
        Shine,
        QDesign
    }

    // VBR header ID for Xing/FhG
    public static class VBRHeaderID
    {
        private const String m_Xing = "Xing";
        private const String m_FhG = "VBRI";

        public static String Xing { get { return m_Xing; } }
        public static String FhG { get { return m_FhG; } }
    }

    // VBR Vendor ID strings
    public static class VBRVendorID
    {
        private const String m_LAME = "LAME";
        private const String m_GoGoNew = "GOGO";
        private const String m_GoGoOld = "MPGE";

        public static String LAME { get { return m_LAME; } }
        public static String GoGoNew { get { return m_GoGoNew; } }
        public static String GoGoOld { get { return m_GoGoOld; } }
    }

    // Preset guess status
    public enum UsePresetGuess
    {
        NotNeeded,
        UseGuess,
        UnableToGuess
    };

    public enum LamePreset
    {
        Insane, // --[alt-]preset insane
        Extreme, // --[alt-]preset extreme
        FastExtreme, // --[alt-]preset fast extreme
        Standard, // --[alt-]preset standard
        FastStandard, // --[alt-]preset fast standard
        Medium, // --preset medium
        FastMedium, // --preset medium
        R3mix, // --r3mix
        Studio, // --preset studio
        CD, // --preset cd
        Hifi, // --preset hifi
        Tape, // --preset tape
        Radio, // --preset radio
        FM, // --preset fm
        TapeRadioFM, // --preset tape OR --preset radio OR --preset fm
        // the above three presets generate identical files for
        // versions of lame from 3.90.3 and 3.93 up
        Voice, // --preset voice
        MWUS, // --preset mw-us
        MWEU, // --preset phon+ OR --preset lw OR --preset mw-eu OR --preset sw
        Phone, // --preset phone
        Unknown
    };

    public enum LameVersionGroup
    {
        lvg390_3901_392,
        lvg3902_391,
        lvg393,
        lvg3931_3903up,
        lvg394up,
        None
    };

}
